<?php

namespace CreoCode\ContaoCssHelper\Classes;

use Contao\Database;
use Contao\DataContainer;
use Contao\Input;

use CreoCode\ContaoCssHelper\Models\CssHelperSelectorModel;

class CssHelperCore
{
    /**
     * @param $varValue
     * @param DataContainer $dc
     * @param string $specialName
     * @return bool
     */
    public function saveCssIdCallback($varValue, DataContainer $dc, $specialName = 'cssID')
    {
        if (!$dc->activeRecord) {
            return false;
        }

        $arrCssID = $this->getCssIDValue($dc, $specialName);
        $arrClasses = $this->getClassesFromCssIDAsArray($arrCssID);

        $availableHelperStyles = $this->getAllCssStyleSelectorClassesByTable($dc->table);

        // Remove all known cssHelperSelector classes from cssID classes
        //$cssHelperClassRemoved = array_diff(unserialize($dc->activeRecord->cssHelperSelector), unserialize($varValue));
        $cssHelperClassRemoved = array_diff($arrClasses, $this->getAllCssStyleSelectorClassesByTable($dc->table));
        if (!empty($cssHelperClassRemoved)) {
            $arrClasses = array_diff($arrClasses, $this->getAllCssStyleSelectorClassesByTable($dc->table));
        }

        if ($varValue) {
            $varValue = unserialize($varValue);
        } else {
            $varValue = array();
        }
        
        foreach ($arrClasses as $key => $value) {
            // Search if a manual class is added and present in the css helper styles
            $arrayIndex = array_search($value, $availableHelperStyles);
            if ($arrayIndex) {
                // Add manual class to the css helper styles
                if (!in_array($arrayIndex, $varValue)) {
                    $varValue[] = $arrayIndex;
                }
            }

        }

        $varValue = serialize($varValue);
        // Remove all known cssHelperSelector classes from cssID classes
        //$arrClasses = array_diff($arrClasses, $this->getAllCssStyleSelectorClassesByTable($dc->table));

        // Add all selected classes of CssStyleSelector to the classes of cssID
        $arrCssClassesSelectorIds = $this->convertSerializedCssStyleSelectorToArray($varValue);
        $arrClasses = array_merge($arrClasses, $this->getCssStyleSelectorClassesByIds($arrCssClassesSelectorIds));

        $arrClasses = array_unique($arrClasses);

        $this->saveClassesToCssID($arrClasses, $dc, $specialName);

        return $varValue;
    }

    /**
     * @param $varValue
     * @param DataContainer $dc
     * @param string $specialName
     * @return bool
     */
    public function saveCssClassCallback($varValue, DataContainer $dc, $specialName = 'cssClass')
    {
        if (!$dc->activeRecord) {
            return false;
        }

        $strCssClasses = $this->getCssClassValue($dc, $specialName);
        $arrClasses = $this->convertClassesStringToArray($strCssClasses);

        // Remove all known cssHelperSelector classes from cssID classes
        $arrClasses = array_diff($arrClasses, $this->getAllCssStyleSelectorClassesByTable($dc->table));

        // Add all selected classes of CssStyleSelector to the classes of cssID
        $arrCssClassesSelectorIds = $this->convertSerializedCssStyleSelectorToArray($varValue);
        $arrClasses = array_merge($arrClasses, $this->getCssStyleSelectorClassesByIds($arrCssClassesSelectorIds));

        $arrClasses = array_unique($arrClasses);

        $this->saveClassesToCssClass($arrClasses, $dc, $specialName);

        return $varValue;
    }

    /**
     * onload_callback for the tl_content DCA to inject cssHelperSelector for any regular custom content element.
     * @param  DataContainer $dc
     * @return void
     */
    public function onLoadContentCallback($dc)
    {
        if (!($dc instanceof DataContainer)) {
            return;
        }
        
        // Get the type
        $type = null;
        if (Input::post('FORM_SUBMIT') === $dc->table) {
            $type = Input::post('type');
        }
        else {
            if ($dc->activeRecord) {
                $type = $dc->activeRecord->type;
            } else {
                $table = $dc->table;
                $id = $dc->id;

                if (Input::get('target'))
                {
                    $table = \explode('.', Input::get('target'), 2)[0];
                    $id = (int) \explode('.', Input::get('target'), 3)[2];
                }

                if ($table && $id) {
                    $record = Database::getInstance()->prepare("SELECT * FROM {$table} WHERE id=?")->execute($id);
                    if ($record->next()) {
                        $type = $record->type;
                    }
                }
            }
        }

        // The palette might not exist
        if (\array_key_exists($type, $GLOBALS['TL_DCA'][$dc->table]['palettes'])) {

            // Get the palette
            $palette = &$GLOBALS['TL_DCA'][$dc->table]['palettes'][$type];

            // Check if cssID is in the palette and cssHelperSelector is not
            if (\strpos($palette, 'cssID') !== false &&
                \strpos($palette, 'cssHelperSelector') === false) {

                // Add the css style selector
                $palette = \str_replace(',cssID', ',cssHelperSelector,cssID', $palette);
            }
        }
    }

    /**
     * @param $intId
     * @param string $specialName
     * @return string
     */
    protected function getCssIDName($intId, $specialName = 'cssID')
    {
        return $specialName . ((Input::get('act') == 'editAll') ? '_' . $intId : '');
    }

    /**
     * @param DataContainer $dc
     * @param string $specialName
     * @return array
     */
    protected function getCssIDValue(DataContainer $dc, $specialName = 'cssID')
    {
        $arrCssID = Input::post($this->getCssIDName($dc->id, $specialName));

        if ($arrCssID === null) {
            $arrCssID = unserialize($dc->activeRecord->cssID);
        }

        if (!is_array($arrCssID)) {
            $arrCssID = array();
        }

        return $arrCssID;
    }

    /**
     * @param $intId
     * @param string $specialName
     * @return string
     */
    protected function getCssClassName($intId, $specialName = 'cssClass')
    {
        return $specialName . ((Input::get('act') == 'editAll') ? '_' . $intId : '');
    }

    /**
     * @param DataContainer $dc
     * @param string $specialName
     * @return string
     */
    protected function getCssClassValue(DataContainer $dc, $specialName = 'cssClass')
    {
        $strCssClass = Input::post($this->getCssClassName($dc->id, $specialName));

        if ($strCssClass === null) {
            $strCssClass = $dc->activeRecord->cssClass;
        }

        if (!is_string($strCssClass)) {
            $strCssClass = '';
        }

        return $strCssClass;
    }

    /**
     * @param string $strValue
     * @return array
     */
    protected function convertSerializedCssStyleSelectorToArray($strValue)
    {
        $arrIds = unserialize($strValue);

        if (!is_array($arrIds)) {
            $arrIds = array();
        }

        return $arrIds;
    }

    /**
     * @param array $arrClasses
     * @param DataContainer $dc
     * @param string $specialName
     */
    protected function saveClassesToCssID(array $arrClasses, DataContainer $dc, $specialName = 'cssID')
    {
        $strCssIDName = $this->getCssIDName($dc->id, $specialName);

        $arrPostedCssID = Input::post($strCssIDName);
        $arrPostedCssID[1] = implode(' ', $arrClasses);
        $arrPostedCssID[1] = str_replace('  ', ' ', $arrPostedCssID[1]);
        $arrPostedCssID[1] = trim($arrPostedCssID[1]);

        $dc->activeRecord->cssID = serialize($arrPostedCssID);
        Input::setPost($strCssIDName, $arrPostedCssID);

        $objDatabase = Database::getInstance();
        $objDatabase->prepare("UPDATE $dc->table SET " . $specialName . "=? WHERE id=?")
            ->execute(serialize($arrPostedCssID), $dc->id);
    }

    /**
     * @param array $arrClasses
     * @param DataContainer $dc
     * @param string $specialName
     */
    protected function saveClassesToCssClass(array $arrClasses, DataContainer $dc, $specialName = 'cssClass')
    {
        $strCssClassName = $this->getCssClassName($dc->id, $specialName);

        $strClasses = implode(' ', $arrClasses);
        $strClasses = str_replace('  ', ' ', $strClasses);
        $strClasses = trim($strClasses);

        $dc->activeRecord->cssClass = $strClasses;
        Input::setPost($strCssClassName, $strClasses);

        $objDatabase = Database::getInstance();
        $objDatabase->prepare("UPDATE $dc->table SET " . $specialName . "=? WHERE id=?")
            ->execute($strClasses, $dc->id);
    }

    /**
     * @param array $arrCssID
     * @return array
     */
    protected function getClassesFromCssIDAsArray(array $arrCssID)
    {
        list($strId, $strClasses) = $arrCssID;

        $arrClasses = $this->convertClassesStringToArray($strClasses);

        return $arrClasses;
    }

    /**
     * @param array $arrIds
     * @return array
     */
    protected function getCssStyleSelectorClassesByIds(array $arrIds)
    {
        if (empty($arrIds)) {
            return array();
        }

        $arrClasses = CssHelperSelectorModel::findCssClassesByIds($arrIds);

        return $this->convertCombinedClassesToSingleClasses($arrClasses);
    }

    /**
     * @param string $strTable
     * @return array
     */
    protected function getAllCssStyleSelectorClassesByTable($strTable)
    {
        if (empty($strTable)) {
            return array();
        }

        $strType = strtolower(substr($strTable, 3));

        $arrClasses = CssHelperSelectorModel::findAllCssHelperClasses($strType);
        //$arrClasses = $this->convertCombinedClassesToSingleClasses($arrClasses);

        return $arrClasses;
    }

    /**
     * @param array $arrClasses
     * @return array
     */
    protected function convertCombinedClassesToSingleClasses(array $arrClasses)
    {
        $arrSingleClasses = array();

        if (is_array($arrClasses)) {
            foreach ($arrClasses as $key => $value) {
                $arrSingleClasses = array_merge($arrSingleClasses, $this->convertClassesStringToArray($value));
            }
        }

        $arrSingleClasses = array_unique($arrSingleClasses);

        return $arrSingleClasses;
    }

    /**
     * @param string $strClasses
     * @return array
     */
    protected function convertClassesStringToArray($strClasses)
    {
        $arrClasses = explode(' ', $strClasses);

        if (empty($arrClasses)) {
            $arrClasses = array();
        }

        return $arrClasses;
    }
}
