<?php

namespace CreoCode\ContaoCssHelper\Models;

use Contao\Database;
use Contao\Model;
use Contao\Input;

class CssHelperSelectorModel extends Model
{
    const TYPE_ARTICLE = 'article';
    const TYPE_CALENDAR_EVENTS = 'calendarEvent';
    const TYPE_CONTENT = 'content';
    const TYPE_FORM = 'form';
    const TYPE_FORM_FIELD = 'formField';
    const TYPE_LAYOUT = 'layout';
    const TYPE_NEWS = 'news';
    const TYPE_MODEL = 'module';
    const TYPE_PAGE = 'page';

    /**
     * Name of the table
     * @var string
     */
    protected static $strTable = 'tl_css_helper_style';
    protected static $strTableParent = 'tl_css_helper_set';

    public static function getSupportedTypes()
    {
        return array(
            self::TYPE_ARTICLE,
            self::TYPE_CALENDAR_EVENTS,
            self::TYPE_CONTENT,
            self::TYPE_FORM,
            self::TYPE_FORM_FIELD,
            self::TYPE_LAYOUT,
            self::TYPE_NEWS,
            self::TYPE_MODEL,
            self::TYPE_PAGE
        );
    }

    /**
     * @param array $arrIds
     * @return array
     */
    public static function findCssClassesByIds(array $arrIds)
    {
        $t = self::$strTable;
        $objDatabase = Database::getInstance();

        $objCssStyleSelector = $objDatabase->prepare("SELECT cssClasses FROM $t WHERE id IN(". implode(',', array_map('intval', array_unique($arrIds))) .")")->execute();

        return $objCssStyleSelector->fetchEach('cssClasses');
    }

    /**
     * @param $strType
     * @return array
     */
    public static function findAllCssHelperClasses($strType)
    {
        if (!in_array($strType, self::getSupportedTypes())) {
            return array();
        }

        $t = self::$strTable;
        $objDatabase = Database::getInstance();

        $objCssStyleSelector = $objDatabase
            ->prepare("SELECT cssClasses,id FROM $t")
            ->execute();

        return $objCssStyleSelector->fetchEach('cssClasses');
    }

    /**
     * @param $strType
     * @return array
     */
    public static function getCssHelperStyles($strType, $returnType='options')
    {

        if (!in_array($strType, self::getSupportedTypes())) {
            return array();
        }

        $t = self::$strTable;
        $tp = self::$strTableParent;
        $elementID = Input::get('id');

        $objDatabase = Database::getInstance();

        if ($strType === 'content') {
            $objElement = $objDatabase
                ->prepare("SELECT type FROM tl_content where id=?")
                ->limit(1)
                ->execute($elementID)
                ->fetchAssoc();
        } else {
            $objElement['type'] = $strType;
        }


        $objCssSetSelector = $objDatabase
            ->prepare("SELECT id,title AS title FROM $tp ORDER BY title ASC")
            ->execute()
            ->fetchAllAssoc();

        $objCssStyleSelector = $objDatabase
            ->prepare("SELECT pid,id,title,cssClasses,inherit,elements,modules FROM $t WHERE published=? ORDER BY title ASC")
            ->execute(1)
            ->fetchAllAssoc();

        $options = array();
        $data = array();
        foreach ($objCssSetSelector as $valueSet) {
            $options[$valueSet['title']] = array();
            $data[$valueSet['title']] = array();
            foreach ($objCssStyleSelector as $value) {


                if ($value['pid'] === $valueSet['id']) {

                    if ($value['inherit'] === 'all') {
                        $options[$valueSet['title']][$value['id']] = $value['title'];
                        $data[$valueSet['title']][$value['id']] = [
                            'title' => $value['title'],
                            'classes' => $value['cssClasses'],
                        ];
                    }

                    $arrElements = unserialize($value['elements']);
                    if ($value['inherit'] === 'custom' && $arrElements) {
                        if(in_array($objElement['type'], $arrElements)) {
                            $options[$valueSet['title']][$value['id']] = $value['title'];
                            $data[$valueSet['title']][$value['id']] = [
                                'title' => $value['title'],
                                'classes' => $value['cssClasses'],
                            ];
                        }
                    }


                    $arrModules = unserialize($value['modules']);
                    if ($value['inherit'] === 'custom' && $arrModules) {
                        if(in_array($objElement['type'], $arrModules)) {
                            $options[$valueSet['title']][$value['id']] = $value['title'];
                            $data[$valueSet['title']][$value['id']] = [
                                'title' => $value['title'],
                                'classes' => $value['cssClasses'],
                            ];
                        }
                    }

                }
            }
        }

        if($returnType === 'data') {
            return $data;
        }

        return $options;
    }



    /**
     * @param $strType
     * @return array
     */
    public static function getCssHelperStylesData($strType)
    {
        $data = self::getCssHelperStyles($strType, 'data');
        return '<script id="ctrl_cssHelperData" type="application/json">'.json_encode($data).'</script>';
    }

    
}
